@@ -31,6 +31,7 @@
 #include "panel-profile.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
+#include "panel-properties-dialog.h"
 
 #define SMALL_ICON_SIZE 20
 
@@ -100,6 +101,8 @@ panel_applet_lock (GtkWidget  *menuitem,
 
 	if (info->move_item)
 		gtk_widget_set_sensitive (info->move_item, !locked);
+	if (info->remove_item)
+		gtk_widget_set_sensitive (info->remove_item, !locked);
 }
 
 static void
@@ -127,16 +130,8 @@ panel_applet_clean (AppletInfo *info)
 {
 	g_return_if_fail (info != NULL);
 
-	switch (info->type) {
-	case PANEL_OBJECT_LAUNCHER:
+	if (info->type == PANEL_OBJECT_LAUNCHER)
 		panel_launcher_delete (info->data);
-		break;
-	case PANEL_OBJECT_DRAWER:
-		panel_drawer_delete (info->data);
-		break;
-	default:
-		break;
-	}
 
 	if (info->widget) {
 		GtkWidget *widget = info->widget;
@@ -185,10 +180,8 @@ applet_callback_callback (GtkWidget      *widget,
 	case PANEL_OBJECT_DRAWER: 
 		if (strcmp (menu->name, "properties")==0) {
 			Drawer *drawer = menu->info->data;
-			g_assert(drawer);
-#ifdef FIXME_FOR_NEW_CONFIG
-			panel_config (drawer->toplevel);
-#endif
+
+			panel_properties_dialog_present (drawer->toplevel);
 		} else if (strcmp (menu->name, "help") == 0) {
 			panel_show_help (screen, "wgospanel.xml", "gospanel-18");
 		}
@@ -338,7 +331,7 @@ setup_an_item (AppletUserMenu *menu,
 	if (menu->stock_item && menu->stock_item [0])
 		image = gtk_image_new_from_stock (menu->stock_item, GTK_ICON_SIZE_MENU);
 
-	setup_menuitem (menu->menuitem, GTK_ICON_SIZE_MENU, image, menu->text);
+	setup_menuitem (menu->menuitem, GTK_ICON_SIZE_MENU, image, menu->text, FALSE);
 
 	if(submenu)
 		gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu->menuitem);
@@ -354,7 +347,7 @@ setup_an_item (AppletUserMenu *menu,
 	/* if the item is a submenu and doesn't have it's menu
 	   created yet*/
 	} else if (!menu->submenu) {
-		menu->submenu = panel_menu_new ();
+		menu->submenu = panel_create_menu ();
 	}
 
 	if(menu->submenu) {
@@ -416,7 +409,7 @@ add_to_submenus (AppletInfo *info,
 	}
 	
 	if (s_menu->submenu == NULL) {
-		s_menu->submenu = panel_menu_new ();
+		s_menu->submenu = panel_create_menu ();
 		/*a more elegant way to do this should be done
 		  when I don't want to go to sleep */
 		if (s_menu->menuitem != NULL) {
@@ -441,7 +434,7 @@ panel_applet_create_menu (AppletInfo  *info,
 	GtkWidget *menuitem;
 	GList     *l;
 
-	menu = g_object_ref (panel_menu_new ());
+	menu = g_object_ref (panel_create_menu ());
 	gtk_object_sink (GTK_OBJECT (menu));
 
 	/* connect the deactivate signal, so that we can "re-allow" 
@@ -480,15 +473,21 @@ panel_applet_create_menu (AppletInfo  *info,
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
 						  GTK_ICON_SIZE_MENU);
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image , _("_Remove From Panel"));
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image , _("_Remove From Panel"), FALSE);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (applet_remove_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		gtk_widget_set_sensitive (menuitem, removable);
+		gtk_widget_set_sensitive (menuitem, !locked && removable);
+
+		g_assert (info->remove_item == NULL);
+
+		info->remove_item = menuitem;
+		g_object_add_weak_pointer (G_OBJECT (menuitem),
+					   (gpointer *) &info->remove_item);
 
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, locked ? _("Un_lock") : _("_Lock"));
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, locked ? _("Un_lock") : _("_Lock"), FALSE);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (panel_applet_lock), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
@@ -496,7 +495,7 @@ panel_applet_create_menu (AppletInfo  *info,
 		
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
-		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, _("_Move"));
+		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, _("_Move"), FALSE);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (move_applet_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
@@ -676,7 +675,8 @@ typedef struct {
 	guint            locked : 1;
 } PanelAppletToLoad;
 
-static GSList *panel_applets_to_load = NULL;
+static GSList  *panel_applets_to_load = NULL;
+static gboolean panel_applet_have_load_idle = FALSE;
 
 static void
 free_applet_to_load (PanelAppletToLoad *applet)
@@ -690,6 +690,18 @@ free_applet_to_load (PanelAppletToLoad *applet)
 	g_free (applet);
 }
 
+gboolean
+panel_applet_on_load_queue (const char *id)
+{
+	GSList *li;
+	for (li = panel_applets_to_load; li != NULL; li = li->next) {
+		PanelAppletToLoad *applet = li->data;
+		if (strcmp (applet->id, id) == 0)
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static gboolean
 panel_applet_load_idle_handler (gpointer dummy)
 {
@@ -698,8 +710,10 @@ panel_applet_load_idle_handler (gpointer dummy)
 	PanelWidget       *panel_widget;
 	GSList            *l;
 
-	if (!panel_applets_to_load)
+	if (!panel_applets_to_load) {
+		panel_applet_have_load_idle = FALSE;
 		return FALSE;
+	}
 
 	for (l = panel_applets_to_load; l; l = l->next) {
 		applet = l->data;
@@ -715,6 +729,7 @@ panel_applet_load_idle_handler (gpointer dummy)
 			free_applet_to_load (l->data);
 		g_slist_free (panel_applets_to_load);
 		panel_applets_to_load = NULL;
+		panel_applet_have_load_idle = FALSE;
 		return FALSE;
 	}
 
@@ -799,6 +814,7 @@ panel_applet_queue_applet_to_load (char            *id,
 
 	if (!toplevel_id) {
 		g_warning ("No toplevel on which to load object '%s'\n", id);
+		g_free (id);
 		return;
 	}
 
@@ -835,7 +851,10 @@ panel_applet_load_queued_applets (void)
 	panel_applets_to_load = g_slist_sort (panel_applets_to_load,
 					      (GCompareFunc) panel_applet_compare);
 
-	g_idle_add (panel_applet_load_idle_handler, NULL);
+	if ( ! panel_applet_have_load_idle) {
+		g_idle_add (panel_applet_load_idle_handler, NULL);
+		panel_applet_have_load_idle = TRUE;
+	}
 }
 
 static G_CONST_RETURN char *
@@ -923,10 +942,16 @@ panel_applet_save_position (AppletInfo *applet_info,
 		gconf_client_set_string (client, key, toplevel_id, NULL);
 	g_free (old_toplevel_id);
 
+	/* Note: changing some properties of the panel that may not be locked down
+	   (e.g. background) can change the state of the "panel_right_stick" and
+	   "position" properties of an applet that may in fact be locked down.
+	   So check if these are writable before attempting to write them */
+
 	right_stick = panel_is_applet_right_stick (applet_info->widget) ? 1 : 0;
 	key = panel_gconf_full_key (
 			key_type, profile, id, "panel_right_stick");
-	if (gconf_client_get_bool (client, key, NULL) ? 1 : 0 != right_stick)
+	if (gconf_client_key_is_writable (client, key, NULL) &&
+	    (gconf_client_get_bool (client, key, NULL) ? 1 : 0) != right_stick)
 		gconf_client_set_bool (client, key, right_stick, NULL);
 
 	position = panel_applet_get_position (applet_info);
@@ -934,7 +959,8 @@ panel_applet_save_position (AppletInfo *applet_info,
 		position = panel_widget->size - position;
 
 	key = panel_gconf_full_key (key_type, profile, id, "position");
-	if (gconf_client_get_int (client, key, NULL) != position)
+	if (gconf_client_key_is_writable (client, key, NULL) &&
+	    gconf_client_get_int (client, key, NULL) != position)
 		gconf_client_set_int (client, key, position, NULL);
 	
 	locked = panel_widget_get_applet_locked (panel_widget, applet_info->widget) ? 1 : 0;
@@ -952,6 +978,24 @@ panel_applet_get_id (AppletInfo *info)
 	return info->id;
 }
 
+const char *
+panel_applet_get_id_by_widget (GtkWidget *applet_widget)
+{
+	GSList *l;
+
+	if (!applet_widget)
+		return NULL;
+
+	for (l = registered_applets; l; l = l->next) {
+		AppletInfo *info = l->data;
+
+		if (info->widget == applet_widget)
+			return info->id;
+	}
+
+	return NULL;
+}
+
 AppletInfo *
 panel_applet_get_by_id (const char *id)
 {
@@ -1011,13 +1055,15 @@ panel_applet_register (GtkWidget       *applet,
 					  GDK_POINTER_MOTION_HINT_MASK));
 
 	info = g_new0 (AppletInfo, 1);
-	info->type = type;
-	info->widget = applet;
-	info->menu = NULL;
-	info->data = data;
+	info->type         = type;
+	info->widget       = applet;
+	info->menu         = NULL;
+	info->data         = data;
 	info->data_destroy = data_destroy;
-	info->user_menu = NULL;
-	info->move_item = NULL;
+	info->user_menu    = NULL;
+	info->move_item    = NULL;
+	info->remove_item  = NULL;
+	info->id           = g_strdup (id);
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
@@ -1084,8 +1130,6 @@ panel_applet_register (GtkWidget       *applet,
 	back_change (info, panel);
 #endif
 
-	if (id) info->id = g_strdup (id);
-
 	if (type != PANEL_OBJECT_BONOBO)
 		gtk_widget_grab_focus (applet);
 	else
