@@ -871,23 +871,31 @@ egg_recent_model_open_file (EggRecentModel *model)
 static gboolean
 egg_recent_model_lock_file (FILE *file)
 {
+#ifdef HAVE_LOCKF
 	int fd;
 
 	rewind (file);
 	fd = fileno (file);
 
 	return lockf (fd, F_LOCK, 0) == 0 ? TRUE : FALSE;
+#else
+	return FALSE; /* FIXME */
+#endif
 }
 
 static gboolean
 egg_recent_model_unlock_file (FILE *file)
 {
+#ifdef HAVE_LOCKF
 	int fd;
 
 	rewind (file);
 	fd = fileno (file);
 
 	return lockf (fd, F_ULOCK, 0) < 0 ? FALSE : TRUE;
+#else
+	return FALSE; /* FIXME */
+#endif
 }
 
 static void
@@ -1211,20 +1219,26 @@ egg_recent_model_add_full (EggRecentModel * model, EggRecentItem *item)
 	GList *list = NULL;
 	gboolean ret = FALSE;
 	gboolean updated = FALSE;
+	char *uri;
 	time_t t;
-	gchar *uri;
 	
 	g_return_val_if_fail (model != NULL, FALSE);
 	g_return_val_if_fail (EGG_IS_RECENT_MODEL (model), FALSE);
 
+	uri = egg_recent_item_get_uri (item);
+	if (strncmp (uri, "recent-files://", strlen ("recent-files://")) == 0) {
+		g_free (uri);
+		return FALSE;
+	} else {
+		g_free (uri);
+	}
+
 	file = egg_recent_model_open_file (model);
 	g_return_val_if_fail (file != NULL, FALSE);
 
 	time (&t);
 	egg_recent_item_set_timestamp (item, t);
 
-	uri = egg_recent_item_get_uri (item);
-
 	if (egg_recent_model_lock_file (file)) {
 
 		/* read existing stuff */
