@@ -348,6 +348,7 @@ activate_app_def (GtkWidget  *menuitem,
 			panel_error_dialog (menuitem_to_screen (menuitem),
 					    "cannot_launch_entry",
 					    _("Cannot launch entry"),
+					    "%s",
 					    error->message);
 
 			g_clear_error (&error);
@@ -359,6 +360,7 @@ activate_app_def (GtkWidget  *menuitem,
 		panel_error_dialog (menuitem_to_screen (menuitem),
 				    "cannot_load_entry",
 				    _("Cannot load entry"),
+				    "%s",
 				    error->message);
 
 		g_clear_error (&error);
@@ -1058,8 +1060,9 @@ remove_menuitem (GtkWidget *widget, ShowItemMenu *sim)
 		panel_error_dialog (menuitem_to_screen (sim->menuitem),
 				    "cannot_remove_menu_item",
 				    _("Cannot remove menu item %s"),
-				    gnome_vfs_result_to_string (result),
-				    esc);
+				    "%s",
+				    esc,
+				    gnome_vfs_result_to_string (result));
 
 		g_free (esc);
 		return;
@@ -1153,6 +1156,7 @@ add_to_run_dialog (GtkWidget    *widget,
 		panel_error_dialog (menuitem_to_screen (sim->menuitem),
 				    "cannot_load_entry",
 				    _("Cannot load entry"),
+				    "%s",
 				    error->message);
 
 		g_clear_error (&error);
@@ -1179,6 +1183,7 @@ show_help_on (GtkWidget    *widget,
 			panel_error_dialog (screen,
 					    "cannot_show_gnome_kde_help",
 					    _("Cannot display help document"),
+					    "%s",
 					    error->message);
 
 			g_clear_error (&error);
@@ -1191,6 +1196,7 @@ show_help_on (GtkWidget    *widget,
 		panel_error_dialog (screen,
 				    "cannot_load_entry",
 				    _("Cannot load entry"),
+				    "%s",
 				    error->message);
 
 		g_clear_error (&error);
@@ -1216,88 +1222,91 @@ add_app_to_panel (GtkWidget    *item,
 }
 
 
-static void
-add_drawers_from_dir (const char *dirname, const char *name,
-		      int pos, PanelWidget *panel)
+static inline gboolean
+vfs_text_uri_exists (const char *text_uri)
 {
-	QuickDesktopItem *item_info;
-	char *dentry_name;
-	const char *subdir_name;
-	char *pixmap_name;
+	GnomeVFSURI *uri;
+	gboolean     retval;
 
-	dentry_name = g_build_path ("/",
-				    dirname,
-				    ".directory",
-				    NULL);
-	item_info = quick_desktop_item_load_uri (dentry_name,
-						 NULL /* expected_type */,
-						 FALSE /* run_tryexec */);
-	g_free (dentry_name);
+	uri = gnome_vfs_uri_new (text_uri);
+	retval = gnome_vfs_uri_exists (uri);
+	gnome_vfs_uri_unref (uri);
 
-	if (name == NULL)
-		subdir_name = item_info != NULL ? item_info->name : NULL;
-	else
-		subdir_name = name;
-	pixmap_name = item_info != NULL ? item_info->icon : NULL;
-
-	panel_drawer_create (panel->toplevel,
-			     pos,
-			     pixmap_name,
-			     pixmap_name != NULL,
-			     subdir_name);
-
-#ifdef FIXME_FOR_NEW_CONFIG
-	Drawer *drawer;
-	PanelWidget *newpanel;
-	char *filename = NULL;
-	GSList *list, *li;
-
-	newpanel = panel_toplevel_get_panel_widget (drawer->toplevel);
-
-	list = get_mfiles_from_menudir (dirname, NULL /* sorted */);
-	for(li = list; li!= NULL; li = li->next) {
-		MFile *mfile = li->data;
-
-		g_free (filename);
-		filename = g_build_filename (dirname, mfile->name, NULL);
-
-		if ( ! mfile->verified) {
-			continue;
-		}
-
-		if (mfile->is_dir) {
-			add_drawers_from_dir (filename, NULL, G_MAXINT/2,
-					      newpanel);
-			continue;
-		}
-			
-		if ((g_str_has_suffix (mfile->name, ".desktop") ||
-		     g_str_has_suffix (mfile->name, ".kdelnk")) &&
-		    g_file_test (filename, G_FILE_TEST_EXISTS))
-			panel_launcher_create (newpanel->toplevel, G_MAXINT/2, filename);
-	}
-	g_free (filename);
-
-	free_mfile_list (list);
-#endif
+	return retval;
 }
 
-/*add a drawer with the contents of a menu to the panel*/
 static void
-add_menudrawer_to_panel(GtkWidget *widget, gpointer data)
+add_drawers_from_dir (const char  *dirname,
+		      const char  *name,
+		      int          pos,
+		      const char  *toplevel_id)
 {
-	MenuFinfo *mf = data;
-	PanelWidget *panel = menu_get_panel (widget);
-	PanelData *pd;
-	int insertion_pos = -1;
+	QuickDesktopItem *item_info;
+	GSList           *mfiles, *l;
+	const char       *subdir_name;
+	char             *dentry_name;
+	char             *pixmap_name;
+	char             *uri;
+	char             *attached_toplevel_id;
 
-	g_return_if_fail (mf != 0);
+	dentry_name = g_build_path ("/", dirname, ".directory", NULL);
+	item_info = quick_desktop_item_load_uri (dentry_name, NULL, FALSE);
+	g_free (dentry_name);
+
+	subdir_name = name ? name : (item_info ? item_info->name : NULL);
+	pixmap_name = item_info ? item_info->icon : NULL;
+
+	attached_toplevel_id = panel_drawer_create_with_id (toplevel_id,
+							    pos,
+							    pixmap_name,
+							    pixmap_name != NULL,
+							    subdir_name);
+	if (!attached_toplevel_id)
+		return;
+
+	mfiles = get_mfiles_from_menudir (dirname, NULL);
+	for (l = mfiles; l; l = l->next) {
+		MFile *mfile = l->data;
+
+		uri = g_build_filename (dirname, mfile->name, NULL);
+
+		if (mfile->verified) {
+			if (mfile->is_dir)
+				add_drawers_from_dir (uri, NULL, G_MAXINT/2, attached_toplevel_id);
+
+			else if ((g_str_has_suffix (mfile->name, ".desktop") ||
+				  g_str_has_suffix (mfile->name, ".kdelnk")) &&
+				 vfs_text_uri_exists (uri))
+				panel_launcher_create_with_id (attached_toplevel_id, G_MAXINT/2, uri);
+		}
+
+		g_free (uri);
+	}
+
+	free_mfile_list (mfiles);
+	g_free (attached_toplevel_id);
+}
+
+static void
+add_menudrawer_to_panel (GtkWidget *menuitem,
+			 MenuFinfo *mf)
+
+{
+	PanelWidget *panel;
+	PanelData   *pd;
+	int          insertion_pos;
+
+	g_return_if_fail (mf != NULL);
+	
+	panel = menu_get_panel (menuitem);
 
 	pd = g_object_get_data (G_OBJECT (panel->toplevel), "PanelData");
-	if (pd != NULL)
-		insertion_pos = pd->insertion_pos;
+	insertion_pos = pd ? pd->insertion_pos : -1;
 
-	add_drawers_from_dir (mf->menudir, mf->dir_name, insertion_pos, panel);
+	add_drawers_from_dir (mf->menudir,
+			      mf->dir_name,
+			      insertion_pos,
+			      panel_profile_get_toplevel_id (panel->toplevel));
 }
 
 static void
@@ -1351,13 +1360,21 @@ restore_grabs(GtkWidget *w, gpointer data)
 
 	/*only grab if this HAD a grab before*/
 	if (xgrab_shell && (GTK_MENU_SHELL (xgrab_shell)->have_xgrab))
-		GTK_MENU_SHELL (xgrab_shell)->have_xgrab = 
-			(gdk_pointer_grab (xgrab_shell->window, TRUE,
-					   GDK_BUTTON_PRESS_MASK |
-					   GDK_BUTTON_RELEASE_MASK |
-					   GDK_ENTER_NOTIFY_MASK |
-					   GDK_LEAVE_NOTIFY_MASK,
-					   NULL, NULL, 0) == 0);
+          {
+	    if (gdk_pointer_grab (xgrab_shell->window, TRUE,
+				  GDK_BUTTON_PRESS_MASK |
+				  GDK_BUTTON_RELEASE_MASK |
+				  GDK_ENTER_NOTIFY_MASK |
+				  GDK_LEAVE_NOTIFY_MASK,
+				  NULL, NULL, 0) == 0)
+              {
+		if (gdk_keyboard_grab (xgrab_shell->window, TRUE,
+				       GDK_CURRENT_TIME) == 0)
+		  GTK_MENU_SHELL (xgrab_shell)->have_xgrab = TRUE;
+		else
+		  gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	      }
+         }
 	
 	gtk_grab_add (GTK_WIDGET (menu));
 }
@@ -2771,7 +2788,7 @@ create_applets_menu (GtkWidget             *menu,
 static void
 create_new_panel (GtkWidget *menuitem)
 {
-	panel_profile_create_toplevel ();
+	panel_profile_create_toplevel (gtk_widget_get_screen (menuitem));
 }
 
 void
