@@ -83,7 +83,7 @@ panel_menu_button_instance_init (PanelMenuButton      *button,
 
 	button_widget_set_stock_id (BUTTON_WIDGET (button), PANEL_STOCK_MAIN_MENU);
 
-	/* FIXME: set a tooltip */
+	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button), _("Main Menu"), NULL);
 }
 
 static void
@@ -97,8 +97,9 @@ panel_menu_button_finalize (GObject *object)
 	button->priv->gconf_notify = 0;
 
 	if (button->priv->menu) {
+		/* detaching the menu will kill our reference */
 		gtk_menu_detach (button->priv->menu);
-		g_object_unref (button->priv->menu);
+		button->priv->menu = NULL;
 	}
 
 	g_free (button->priv->menu_path);
@@ -281,6 +282,12 @@ panel_menu_button_menu_detacher	(GtkWidget *widget,
 
 	g_return_if_fail (button->priv->menu == menu);
 
+	/* just in case someone still owns a reference to the
+	   menu (the menu may be up or some such other nonsense) */
+	g_signal_handlers_disconnect_by_func (button->priv->menu,
+					      G_CALLBACK (panel_menu_button_menu_deactivated),
+					      button);
+
 	/* This is a workaround pending fixing bug #113112 */
 	g_object_set_data (G_OBJECT (button), "gtk-attached-menu", NULL);
 	button->priv->menu = NULL;
@@ -296,16 +303,13 @@ panel_menu_button_popup_menu (PanelMenuButton *button,
 	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
 
 	if (button->priv->menu && menu_need_reread (GTK_WIDGET (button->priv->menu))) {
-		g_object_unref (button->priv->menu);
+		gtk_menu_detach (button->priv->menu);
 		button->priv->menu = NULL;
 	}
 
 	if (!button->priv->menu) {
 		button->priv->menu = panel_menu_button_create_menu (button);
 
-		g_object_ref (button->priv->menu);
-		gtk_object_sink (GTK_OBJECT (button->priv->menu));
-
 		gtk_menu_attach_to_widget (button->priv->menu, 
 					   GTK_WIDGET (button),
 					   panel_menu_button_menu_detacher);
@@ -564,6 +568,7 @@ panel_menu_button_load (const char  *menu_path,
 			       "custom-icon", custom_icon,
 			       "use-menu-path", use_menu_path,
 			       "use-custom-icon", use_custom_icon,
+			       "has-arrow", TRUE,
 			       NULL);
 
 	button->priv->info = panel_applet_register (
@@ -604,7 +609,7 @@ panel_menu_button_get_icon_for_menu_path (PanelMenuButton *button)
 	if (!item)
 		return NULL;
                                                                                                              
-	retval = gnome_desktop_item_find_icon (panel_icon_theme, item->icon, 20, 0);
+	retval = g_strdup (item->icon);
                                                                                                              
 	quick_desktop_item_destroy (item);
 
@@ -617,15 +622,14 @@ panel_menu_button_set_icon (PanelMenuButton *button)
 	char *icon_path = NULL;
 
 	if (button->priv->use_custom_icon)
-		icon_path = button->priv->custom_icon;
+		icon_path = g_strdup (button->priv->custom_icon);
 
 	if (!icon_path)
 		icon_path = panel_menu_button_get_icon_for_menu_path (button);
 
-	if (icon_path)
-		button_widget_set_icon_name (BUTTON_WIDGET (button), icon_path);
-	else
-		button_widget_set_stock_id (BUTTON_WIDGET (button), PANEL_STOCK_MAIN_MENU);
+	button_widget_set_icon_name (BUTTON_WIDGET (button), icon_path);
+
+	g_free (icon_path);
 }
                                                                                                              
 void
@@ -648,7 +652,7 @@ panel_menu_button_set_menu_path (PanelMenuButton *button,
 		button->priv->menu_path = g_strdup (menu_path);
 
 	if (button->priv->menu)
-		g_object_unref (button->priv->menu);
+		gtk_menu_detach (button->priv->menu);
 	button->priv->menu = NULL;
 
 	panel_menu_button_set_icon (button);
@@ -683,7 +687,7 @@ panel_menu_button_set_use_menu_path (PanelMenuButton *button,
 	button->priv->use_menu_path = use_menu_path;
 
 	if (button->priv->menu)
-		g_object_unref (button->priv->menu);
+		gtk_menu_detach (button->priv->menu);
 	button->priv->menu = NULL;
 
 	panel_menu_button_set_icon (button);
