@@ -35,6 +35,8 @@
 #include "panel-gconf.h"
 #include "panel.h"
 #include "panel-widget.h"
+#include "panel-util.h"
+#include "panel-multiscreen.h"
 
 typedef struct {
 	GdkScreen       *screen;
@@ -634,6 +636,118 @@ TOPLEVEL_GET_SET_FUNCS ("background/fit",     background, bool, fit,
 TOPLEVEL_GET_SET_FUNCS ("background/stretch", background, bool, stretch,        gboolean)
 TOPLEVEL_GET_SET_FUNCS ("background/rotate",  background, bool, rotate,         gboolean)
 
+static const char *
+panel_profile_get_attached_object_key (PanelToplevel *toplevel,
+				       const char    *key)
+{
+	GtkWidget  *attach_widget;
+	const char *id;
+
+	attach_widget = panel_toplevel_get_attach_widget (toplevel);
+
+	id = panel_applet_get_id_by_widget (attach_widget);
+
+	if (!id)
+		return NULL;
+
+	return panel_gconf_full_key (PANEL_GCONF_OBJECTS, current_profile, id, key);
+}
+
+void
+panel_profile_set_attached_custom_icon (PanelToplevel *toplevel,
+					const char    *custom_icon)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "use_custom_icon");
+	if (key)
+		gconf_client_set_bool (client, key, custom_icon != NULL, NULL);
+
+	key = panel_profile_get_attached_object_key (toplevel, "custom_icon");
+	if (key)
+		gconf_client_set_string (client, key, sure_string (custom_icon), NULL);
+}
+
+char *
+panel_profile_get_attached_custom_icon (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "use_custom_icon");
+	if (!key || !gconf_client_get_bool (client, key, NULL))
+		return NULL;
+
+	key = panel_profile_get_attached_object_key (toplevel, "custom_icon");
+
+	return key ? gconf_client_get_string (client, key, NULL) : NULL;
+}
+
+gboolean
+panel_profile_is_writable_attached_custom_icon (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "use_custom_icon");
+	if (!key)
+		return TRUE;
+
+	if (!gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	key = panel_profile_get_attached_object_key (toplevel, "custom_icon");
+
+	return key ? gconf_client_key_is_writable (client, key, NULL) : TRUE;
+}
+
+void
+panel_profile_set_attached_tooltip (PanelToplevel *toplevel,
+				    const char    *tooltip)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "tooltip");
+	if (key)
+		gconf_client_set_string (client, key, tooltip, NULL);
+}
+
+char *
+panel_profile_get_attached_tooltip (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "tooltip");
+
+	return key ? gconf_client_get_string (client, key, NULL) : NULL;
+}
+
+gboolean
+panel_profile_is_writable_attached_tooltip (PanelToplevel *toplevel)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_attached_object_key (toplevel, "tooltip");
+
+	return key ? gconf_client_key_is_writable (client, key, NULL) : TRUE;
+}
+
 static PanelBackgroundType
 get_background_type (GConfClient *client,
 		     const char  *toplevel_dir)
@@ -673,6 +787,9 @@ get_background_color (GConfClient *client,
 	error = NULL;
 	key = panel_gconf_sprintf ("%s/background/color", toplevel_dir);
 	color_str = gconf_client_get_string (client, key, &error);
+	color->gdk.red   = 0;
+	color->gdk.green = 0;
+	color->gdk.blue  = 0;
 	if (error) {
 		g_warning (_("Error reading GConf string value '%s': %s"),
 			   key, error->message);
@@ -1096,7 +1213,7 @@ panel_profile_background_change_notify (GConfClient   *client,
 		if (value->type == GCONF_VALUE_BOOL)
 			panel_background_set_stretch (background,
 						      gconf_value_get_bool (value));
-	} else if (!strcmp (key, "fit")) {
+	} else if (!strcmp (key, "rotate")) {
 		if (value->type == GCONF_VALUE_BOOL)
 			panel_background_set_rotate (background,
 						     gconf_value_get_bool (value));
@@ -1217,10 +1334,98 @@ panel_profile_list_is_writable (PanelGConfKeyType type)
 	return gconf_client_key_is_writable (client, key, NULL);
 }
 
-void
-panel_profile_create_toplevel (void)
+static gboolean
+panel_profile_find_empty_spot (GdkScreen *screen,
+			       PanelOrientation *orientation,
+			       int *monitor)
 {
-	panel_profile_add_to_list (PANEL_GCONF_TOPLEVELS, NULL);
+	GSList *li;
+	int i;
+	int *filled_spots;
+	gboolean found_a_spot = FALSE;
+
+	*monitor = 0;
+	*orientation = PANEL_ORIENTATION_TOP;
+
+	filled_spots = g_new0 (int, panel_multiscreen_monitors (screen));
+
+	for (li = panel_toplevel_list_toplevels (); li != NULL; li = li->next) {
+		PanelToplevel *toplevel = li->data;
+		GdkScreen *toplevel_screen = gtk_window_get_screen (GTK_WINDOW (toplevel));
+		int toplevel_monitor = panel_toplevel_get_monitor (toplevel);
+
+		if (toplevel_screen != screen ||
+		    panel_toplevel_get_is_attached (toplevel) ||
+		    toplevel_monitor < 0)
+			continue;
+
+		filled_spots[toplevel_monitor] |= panel_toplevel_get_orientation (toplevel);
+	}
+
+	for (i = 0; i < panel_multiscreen_monitors (screen); i++) {
+		/* These are ordered based on "priority" of the
+		   orientation when picking it */
+		if ( ! (filled_spots[i] & PANEL_ORIENTATION_TOP)) {
+			*orientation = PANEL_ORIENTATION_TOP;
+			*monitor = i;
+			found_a_spot = TRUE;
+			break;
+		} else if ( ! (filled_spots[i] & PANEL_ORIENTATION_BOTTOM)) {
+			*orientation = PANEL_ORIENTATION_BOTTOM;
+			*monitor = i;
+			found_a_spot = TRUE;
+			break;
+		} else if ( ! (filled_spots[i] & PANEL_ORIENTATION_RIGHT)) {
+			*orientation = PANEL_ORIENTATION_RIGHT;
+			*monitor = i;
+			found_a_spot = TRUE;
+			break;
+		} else if ( ! (filled_spots[i] & PANEL_ORIENTATION_LEFT)) {
+			*orientation = PANEL_ORIENTATION_LEFT;
+			*monitor = i;
+			found_a_spot = TRUE;
+			break;
+		}
+	}
+
+	g_free (filled_spots);
+
+	return found_a_spot;
+}
+
+void
+panel_profile_create_toplevel (GdkScreen *screen)
+{
+	GConfClient     *client;
+	const char      *key;
+	char            *id;
+	char            *dir;
+	PanelOrientation orientation;
+	int              monitor;
+       
+	g_return_if_fail (screen != NULL);
+
+	client = panel_gconf_get_client ();
+
+	id = panel_profile_find_new_id (PANEL_GCONF_TOPLEVELS);
+
+	dir = g_strdup_printf (PANEL_CONFIG_DIR "/%s/toplevels/%s", current_profile, id);
+	panel_gconf_associate_schemas_in_dir (client, dir, PANEL_SCHEMAS_DIR "/toplevels");
+	g_free (dir);
+
+	key = panel_gconf_full_key (PANEL_GCONF_TOPLEVELS, current_profile, id, "screen");
+	gconf_client_set_int (client, key, gdk_screen_get_number (screen), NULL);
+
+	if (panel_profile_find_empty_spot (screen, &orientation, &monitor)) {
+		key = panel_gconf_full_key (PANEL_GCONF_TOPLEVELS, current_profile, id, "monitor");
+		gconf_client_set_int (client, key, monitor, NULL);
+
+		key = panel_gconf_full_key (PANEL_GCONF_TOPLEVELS, current_profile, id, "orientation");
+		gconf_client_set_string (client, key, panel_profile_map_orientation (orientation), NULL);
+	}
+	
+	/* takes ownership of id */
+	panel_profile_add_to_list (PANEL_GCONF_TOPLEVELS, id);
 }
 
 static void
@@ -1584,6 +1789,7 @@ panel_profile_load_object (GConfClient       *client,
 	type_string = gconf_client_get_string (client, key, NULL);
         
 	if (!panel_profile_map_object_type_string (type_string, &object_type)) {
+		g_free (id);
 		g_free (type_string);
 		return;
 	}
@@ -1719,14 +1925,14 @@ panel_profile_load_added_ids (GConfClient            *client,
 	}
 
 	for (l = added_ids; l; l = l->next) {
-		char *toplevel_id = l->data;
+		char *id = l->data;
 
 		if (!profile_dir)
 			profile_dir = gconf_concat_dir_and_key (
-						PANEL_CONFIG_DIR, current_profile);
+				PANEL_CONFIG_DIR, current_profile);
 
-		/* takes ownership of toplevel_id */
-		load_handler (client, profile_dir, type, toplevel_id);
+		/* takes ownership of id */
+		load_handler (client, profile_dir, type, id);
 	}
 
 	g_slist_free (added_ids);
@@ -1771,7 +1977,7 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 				       GConfEntry  *entry)
 {
 	GConfValue *value;
-	GSList     *existing_toplevels;
+	GSList     *l, *existing_toplevels;
 	GSList     *toplevel_ids;
 
 	if (!(value = gconf_entry_get_value (entry)))
@@ -1785,7 +1991,16 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 
 	toplevel_ids = gconf_value_get_list (value);
 
-	existing_toplevels = panel_toplevel_list_toplevels ();
+	existing_toplevels = NULL;
+	for (l = panel_toplevel_list_toplevels (); l; l = l->next) {
+		PanelToplevel *toplevel = l->data;
+
+		/* Attached toplevels aren't on the id list */
+		if (panel_toplevel_get_is_attached (toplevel))
+			continue;
+
+		existing_toplevels = g_slist_prepend (existing_toplevels, toplevel);
+	}
 
 	panel_profile_load_added_ids (client,
 				      PANEL_GCONF_TOPLEVELS,
@@ -1801,6 +2016,8 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 					  toplevel_ids,
 					  (PanelProfileGetIdFunc) panel_profile_get_toplevel_id,
 					  (PanelProfileDestroyFunc) panel_profile_destroy_toplevel);
+
+	g_slist_free (existing_toplevels);
 }
 
 static void
