@@ -110,6 +110,10 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 {
 	GList *l;
 
+	if (dialog->file_sel != NULL)
+		gtk_widget_destroy (GTK_WIDGET (dialog->file_sel));
+	dialog->file_sel = NULL;
+
 	g_object_unref (dialog->program_list_box);
 	
 	g_slist_foreach (dialog->add_icon_paths, (GFunc) gtk_tree_path_free, NULL);
@@ -406,6 +410,9 @@ panel_run_dialog_response (PanelRunDialog *dialog,
 			   int             response,
 			   GtkWidget      *run_dialog)
 {
+
+	dialog->completion_started = FALSE;
+
 	switch (response) {
 	case GTK_RESPONSE_OK:
 		panel_run_dialog_execute (dialog);
@@ -1154,7 +1161,6 @@ entry_event (GtkEditable    *entry,
 	if (event->type == GDK_KEY_PRESS)
 		dialog->use_program_list = FALSE;
 
-	dialog->completion_started = FALSE;
 
 	/* tab completion */
 	if (event->type == GDK_KEY_PRESS &&
@@ -1183,9 +1189,13 @@ entry_event (GtkEditable    *entry,
 			temp = gtk_editable_get_chars (entry, 0, pos);
 			prefix = g_strconcat (temp, event->string, NULL);
 			g_free (temp);
-		} else
+		} else if (pos == tmp &&
+			   tmp == strlen (gtk_entry_get_text (GTK_ENTRY (entry)))) {
 			prefix = g_strconcat (gtk_entry_get_text (GTK_ENTRY (entry)),
 					      event->string, NULL);
+		} else {
+			return FALSE;
+		}
 		
 		pos = strlen (prefix);
 		nprefix = NULL;
@@ -1201,10 +1211,13 @@ entry_event (GtkEditable    *entry,
 
 			g_free (nprefix);
 			g_free (prefix);
+			return TRUE;
 		}
+
+		g_free (prefix);
 	}
 	
-	return dialog->completion_started;
+	return FALSE;
 }
 
 static void
